function [a, bias] = lpc(X,Y,C)
% IMPLEMENTS NORMAL LIKNON.
% (take the input data matrix and class labels, and find the hyperplane
%  that separates the data.)
%  Construct a LP Liknon problem, and solve it.
% X is the matrix of data
% Y are the data labels (vector)  +1 or -1
% C is a constant for multiplying the slack variables.

  if (nargin <2) % check correct number of arguments
    help lpc 
  else

    [n, m]= size(X);

    epsilon = 1e-12;
    I=eye(n); 
    % Construct the  matrix
%    fprintf('Constructing ...\n');
    H = zeros(n, 2*m +n+2); 
    for i=1:n
%     the blocks in H are for the u,v, slack vars k, and the
%     split in two b (bp and bn).  All sign-switched by the
%     experiment label sign, Y
%                       u        v        k          bp  bn
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1];  
    end
    b = ones(n,1);  % the margin constant 1
      % vector f is the target function being minimized
    f = ones(2*m,1);  % covers u and v
    f(2*m+1:2*m+n) = C*ones(n,1);  % how C multiplies slacks, k
    f(2*m+n+1) = 0;   % bp
    f(2*m+n+2) = 0;   % bn
    
    vlb = zeros(2*m+n+2,1); % Set all variable bounds >= 0

    % Solve the Optimisation Problem
    %    fprintf('Optimising ...\n');
    st = cputime;
    
    [x,fval,exitflag,output,lambda] = linprog(f, -H, -b,[],[], vlb);

fprintf('LPOPT %g\n',fval);    
   u = x(1:m);
   v = x(m+1:2*m);
   xi = x(2*m+1:2*m + n);  % the slacks, k or xi
   bias = x(2*m + n + 1)-x(2*m+n+2);  % bias = bp - bn
   a = u - v;     % form the a vector
   for i =1 :m
     if(abs(a(i)) < epsilon)  % set to zero really tiny values
        a(i) = 0;
     end
   end

fprintf('LPSum xi : %g\n',sum(abs(xi)));
end    
