function predict(lpresFile, strExamples, ROTATED)
  if nargin < 3
    ROTATED = 0;
  end;
% Matlab program for checking new examples, part of ROBUST LIKNON

% input files are a "lpres" file as output from a prior
% liknon run, and
% strExamples is the file of examples to test,
% and has the exact same format as the input to liknon.

%fprintf('Start PREDICT\n');
% fprintf('using lpres file: %s\n', lpresFile);
% fprintf('using Examples file: %s\n',strExamples);
 
% a lpres file looks like this (without the %)
%Value	2
%b	1
%4	0.33
%81	-.099
%104	.0222

% we don't care about the Value line, and the first
% two lines make it harder to read into matlab numeric
% arrays. so read one time skipping the first two non-
% numeric lines, then read again just to grab b.
 [indexes, a]  = textread(lpresFile,'%d%f','headerlines',2);
 [garbage, ba] = textread(lpresFile,'%s%f','headerlines',1);
 clear garbage;
 b = ba(1);  % want b as scalar.
 clear ba;
 
 allx = load(strExamples);
 if ROTATED > 0 
   allx = allx';
 end;
 onlyParticularX = fex(allx, indexes); %pick out ONLY indexed data columns
 
 %compute "distance" to hyperplane defined by vector a and scalar b.
 %would be true distance if divide by norm(a), but we only care
 %about sign here.
 predictedY = sign(onlyParticularX * a + b); 
 [m,n] = size(predictedY);
 
 for i=1:m
   fprintf('%d\n', predictedY(i));
 end;
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (c) 1997-2002 The Regents of the University of California.
% All rights reserved.
% Permission is hereby granted, without written agreement and without
% license or royalty fees, to use, copy, modify, and distribute this
% software and its documentation for any purpose, provided that the above
% copyright notice and the following two paragraphs appear in all copies
% of this software.
%
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
% FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
% THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
% SUCH DAMAGE.
%
% THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
% PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
