function [a, bias] = rlpc(X,Y,C, RLPS) 
  if (nargin <2) % check correct number of arguments
    help rlpc 
  else
%%%% ROBUST LIKNON, construct problem and solve it.
% Create a data matrix describing the problem, and solve
% using sedumi.
% all methods are formulated in a similar manner, the matrix
% H (which becomes A) is the problem matrix.  from left to
% right the blocks are: u, v, k, bpos, bneg
% each row is a constraint, the methods have different
% numbers of constraints.
% the equation entered into the matrix is
% u - v + k + bpos - bneg - sigmaterm >= 1
% as can be seen from the signs used in making the H matrix.

% It is IMPORTANT to use SPARSE matrices: otherwise lots of
% memory is used.

if round(RLPS.method) <= 1  % SPHERICAL METHOD
%input('before construct spherical wait');
    [n, m]= size(X);
fprintf('SPHERICAL METHOD\n');
fprintf('input size %d %d\n',n,m);
fprintf('RLPC sigMethod = %d %g\n',RLPS.sigMethod, RLPS.delta);
    epsilon = 1e-20;
    I = speye(n); 
    % Construct the  matrix
fprintf('Constructing ...\n');
    H = zeros(n, 2*m +n+3); 
    for i=1:n
     if (RLPS.sigMethod < 0.5)
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, -Y(i)*RLPS.sigma(i)];  
     else
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, -Y(i)];  
     end
    end
%input('after fill H wait');
    b = sparse(ones(2*m,1));
    b = [b;C*ones(n,1)];  
    b = [b;0;0;0]; 
%    f(2*m+1) = 0;

% The inequality constraints      
    A = -sparse(H);
clear H;
clear I;
%input('after clear H wait');
    c = -sparse(ones(n,1));
    K.l = n;
% WE SHOULDN"T need any more of these "non-negative"
%constraints, as sedumi will auto-do that.  HOWEVER
%sedumi does matrix size checking and it has to be
% length(b) = min(size(A)).
% the non-negativity constraints of u,v
   A = [A; -speye(2*m), zeros(2*m,n+3)];
   c=  [c; zeros(2*m,1)];
   K.l = K.l + 2*m; 
%input('after uv wait');
% the non-negativity constraints of slack
   A = [A; zeros(n,2*m), -speye(n), zeros(n,3)];
   c = [c;zeros(n,1)];
   K.l = n + K.l;

% the free variable b
  A = [A; zeros(1,2*m + n), -1, 0, 0];
  c = [c;0];
  K.l = K.l + 1;
  A = [A; zeros(1,2*m + n), 0, -1, 0];
  c = [c;0];
  K.l = K.l +1;

% the quadratic cone
  A = [A;zeros(1,2*m + n + 2), -1]; 
  if (RLPS.sigMethod < 0.5)
    A = [A; -speye(m), speye(m), zeros(m,n+3)];
  else
    zap = RLPS.delta * spdiags(RLPS.sigma ,zeros(1,1),m,m);
%[za,zb] = size(zap);
%fprintf('diagsigma %d %d %d\n',za,zb,m);
%fprintf('%f %f\n',RLPS.sigma(5), RLPS.sigma(6));
%fprintf('%f %f\n',zap(5,5), zap(5,6));

    A = [A; -zap, zap, zeros(m,n+3)];
    clear zap;
  end
  c = [c; 0;zeros(m,1)];
  K.q = m +1;
    % Solve the Optimisation Problem
%fprintf('final sizes\n');
%[rr,cc] = size(A);
%fprintf('A %d %d\n',rr,cc);
%[rr,cc] = size(b);
%fprintf('b %d %d\n',rr,cc);
%[rr,cc] = size(c);
%fprintf('c %d %d\n',rr,cc);
%fprintf('%d %d\n', min(size(A)), length(b));

   pars.fid = 0; % quiet sedumi.
   [y,x,info]= sedumi(A,-b,c,K, pars);
fprintf('sedumi done pinf=%d dinf=%d numerr=%d feas=%g\n',info.pinf, info.dinf, info.numerr, info.feasratio);
   x = real(x);
   u = x(1:m);
   v = x(m+1:2*m);
   xi = x(2*m+1:2*m + n);
   bias = x(2*m + n + 1)-x(2*m+n+2);

   a = u - v;
% throw away really small values.
   for i =1 :m
     if(abs(a(i)) < epsilon)
        a(i) = 0;
     end
   end

   sumxi = sum(abs(xi));
   fprintf('Sum xi : %g\n',sumxi);
%fprintf('%d\n',size(xi));
%   if sumxi > 0
%     for ijk = 1 :n
%       if (abs(xi(ijk)) > 0)
%         fprintf(' slack %d %g\n',ijk,xi(ijk));
%       end;
%     end;
%   end;
% END OF SPHERICAL METHOD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif round(RLPS.method) == 2
%input('before construct SPLITT SET TO ONE wait');
    [n, m]= size(X);
fprintf('SPLITT SETTO ONE\n');
fprintf('input size %d %d\n',n,m);
fprintf('sigMethod = %d %g\n',RLPS.sigMethod, RLPS.delta);
    epsilon = 1e-20;
    I = speye(n); 
    % Construct the  matrix
fprintf('Constructing ...\n');
    H = zeros(n, 2*m +n+3); 
% The H matrix blocks are: u v k bpos bneg T
% there are m u's, m v's, (2*m), n k's, and 1 each of bpos bneg T
% therefore the number of columns is: 2*m +n+3.
    for i=1:n
     if (RLPS.sigMethod < 0.5)
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, -Y(i)*RLPS.sigma(i)];
     else
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, -Y(i)];  
     end
    end
%input('after fill H wait');
    b = sparse(ones(2*m,1));
    b = [b;C*ones(n,1)];  
    b = [b;0;0;0]; 
%    f(2*m+1) = 0;

% The inequality constraints      
    A = -sparse(H);
clear H;
clear I;
%input('after clear H wait');
% in this formulation, the "magic constant 1" is not explicit,
%rather it is enforced via the T variable: T >= 1
    c = -sparse(0 * ones(n,1)); % THE MAGIC 1 constant
    K.l = n;
% WE SHOULDN"T need any more of these "non-negative"
%constraints, as sedumi will auto-do that.  HOWEVER
%sedumi does matrix size checking and it has to be
% length(b) = min(size(A)).
% the non-negativity constraints of u,v
   A = [A; -speye(2*m), zeros(2*m,n+3)];
   c=  [c; zeros(2*m,1)];
   K.l = K.l + 2*m; 
%input('after uv wait');
% the non-negativity constraints of slack
   A = [A; zeros(n,2*m), -speye(n), zeros(n,3)];
   c = [c;zeros(n,1)];
   K.l = n + K.l;

% the free variable b
  A = [A; zeros(1,2*m + n), -1, 0, 0];
  c = [c;0];
  K.l = K.l + 1;
  A = [A; zeros(1,2*m + n), 0, -1, 0];
  c = [c;0];
  K.l = K.l +1;
%%%%%
% ADD linear constraint to force t greater than zero.
% this takes the place of the magic constant 1.
  A = [A; zeros(1,2*m +n+2), -1];
  c = [c; -1]; % THE T VALUE
  K.l = K.l + 1;
% now have complete new set of constraints for cone.
%    H = zeros(n, 2*m +n+3); 
%    for i=1:n
%     if (sigMethod < 0.5)
%      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), zeros(1,n), 0, 0, -Y(i)*sigma(i)];
%     else
%      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), zeros(1,n), 0, 0, -Y(i)];  
%     end
%    end
%  A = [A; -sparse(H)];
%  c = [c;zeros(n,1)];
%  K.l = K.l + n;
%clear H;
% the quadratic cone
  A = [A;zeros(1,2*m + n + 2), -1]; 
  if (RLPS.sigMethod < 0.5)
    A = [A; -speye(m), speye(m), zeros(m,n+3)];
  else
    zap = RLPS.delta * spdiags(RLPS.sigma ,zeros(1,1),m,m);
%[za,zb] = size(zap);
%fprintf('diagsigma %d %d %d\n',za,zb,m);
%fprintf('%f %f\n',RLPS.sigma(5), RLPS.sigma(6));
%fprintf('%f %f\n',zap(5,5), zap(5,6));

    A = [A; -zap, zap, zeros(m,n+3)];
    clear zap;
  end
  c = [c; 0;zeros(m,1)];
  K.q = m +1;
    % Solve the Optimisation Problem
    
%fprintf('final sizes\n');
%[rr,cc] = size(A);
%fprintf('A %d %d\n',rr,cc);
%[rr,cc] = size(b);
%fprintf('b %d %d\n',rr,cc);
%[rr,cc] = size(c);
%fprintf('c %d %d\n',rr,cc);
%fprintf('%d %d\n', min(size(A)), length(b));

   pars.fid = 0; % quiet sedumi.
   [y,x,info]= sedumi(A,-b,c,K, pars);
fprintf('sedumi done pinf=%d dinf=%d numerr=%d feas=%g\n',info.pinf, info.dinf, info.numerr, info.feasratio);
   x = real(x);
   u = x(1:m);
   v = x(m+1:2*m);
   xi = x(2*m+1:2*m + n);
   bias = x(2*m + n + 1)-x(2*m+n+2);

   a = u - v;
   for i =1 :m
     if(abs(a(i)) < epsilon)
        a(i) = 0;
     end
   end

fprintf('T= %g\n',x(2*m+n+3));
   sumxi = sum(abs(xi));
   fprintf('Sum xi : %g\n',sumxi);
%fprintf('%d\n',size(xi));
   if sumxi > 0
    if n < 10
     for ijk = 1 :n
       if (abs(xi(ijk)) > 0)
         fprintf(' slack %d %g\n',ijk,xi(ijk));
       end;
     end;
    end;
   end;
%%%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif round(RLPS.method) == 3
%input('before construct SPLIT wait');
    [n, m]= size(X);
fprintf('SPLIT CONSTRAINT METHOD\n');
fprintf('input size %d %d\n',n,m);
fprintf('RLPC sigMethod = %d %g\n',RLPS.sigMethod, RLPS.delta);
    epsilon = 1e-20;
    I = speye(n); 
    % Construct the  matrix
fprintf('Constructing ...\n');
    H = zeros(n, 2*m +n+3); 
    % split method has two full sets of constraint equations,
    %this first set has the sigma term set to zero
    for i=1:n
     if (RLPS.sigMethod < 0.5)
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, 0];%-Y(i)*RLPS.sigma(i)];
     else
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, 0];%-Y(i)];  
     end
%      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), 1 -1];  
    end
%input('after fill H wait');
    b = sparse(ones(2*m,1));
    b = [b;C*ones(n,1)];  
    b = [b;0;0;0]; 
%    f(2*m+1) = 0;

% The inequality constraints      
    A = -sparse(H);
clear H;
%clear I;
%input('after clear H wait');
    c = -sparse(1 * ones(n,1)); % THE MAGIC 1 constant
    K.l = n;
% WE SHOULDN"T need any more of these "non-negative"
%constraints, as sedumi will auto-do that.  HOWEVER
%sedumi does matrix size checking and it has to be
% length(b) = min(size(A)).
% the non-negativity constraints of u,v
   A = [A; -speye(2*m), zeros(2*m,n+3)];
   c=  [c; zeros(2*m,1)];
   K.l = K.l + 2*m; 
%input('after uv wait');
% the non-negativity constraints of slack
   A = [A; zeros(n,2*m), -speye(n), zeros(n,3)];
   c = [c;zeros(n,1)];
   K.l = n + K.l;

% the free variable b
  A = [A; zeros(1,2*m + n), -1, 0, 0];
  c = [c;0];
  K.l = K.l + 1;
  A = [A; zeros(1,2*m + n), 0, -1, 0];
  c = [c;0];
  K.l = K.l +1;
%%%%%
% now the complete second set of constraints for cone, with NO
% magic 1 constant, no slacks (k), and no b, but with sigma..
% u - v + 0*k + 0*bpos - 0*bneg - sigma >= 0
    H = zeros(n, 2*m +n+3); 
    for i=1:n
     if (RLPS.sigMethod < 0.5)
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), zeros(1,n), 0, 0, -Y(i)*RLPS.sigma(i)];
     else
%       H(i,:)  = Y(i)*[ X(i,:), -X(i,:), zeros(1,n), 0, 0, -Y(i)];
% add the slacks back in.  
       H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 0, 0, -Y(i)];  
     end
    end
  A = [A; -sparse(H)];
  c = [c;zeros(n,1)];
  K.l = K.l + n;
clear H;
clear I;
% the quadratic cone
  A = [A;zeros(1,2*m + n + 2), -1]; 
  if (RLPS.sigMethod < 0.5)
    A = [A; -speye(m), speye(m), zeros(m,n+3)];
  else
    zap = RLPS.delta * spdiags(RLPS.sigma ,zeros(1,1),m,m);
%[za,zb] = size(zap);
%fprintf('diagsigma %d %d %d\n',za,zb,m);

    A = [A; -zap, zap, zeros(m,n+3)];
    clear zap;
  end
  c = [c; 0;zeros(m,1)];
  K.q = m +1;
    % Solve the Optimisation Problem
    
%fprintf('final sizes\n');
%[rr,cc] = size(A);
%fprintf('A %d %d\n',rr,cc);
%[rr,cc] = size(b);
%fprintf('b %d %d\n',rr,cc);
%[rr,cc] = size(c);
%fprintf('c %d %d\n',rr,cc);
%fprintf('%d %d\n', min(size(A)), length(b));

   pars.fid = 0; % quiet sedumi.
   [y,x,info]= sedumi(A,-b,c,K, pars);
fprintf('sedumi done pinf=%d dinf=%d numerr=%d feas=%g\n',info.pinf, info.dinf, info.numerr, info.feasratio);
   x = real(x);
   u = x(1:m);
   v = x(m+1:2*m);
   xi = x(2*m+1:2*m + n);
   bias = x(2*m + n + 1)-x(2*m+n+2);

   a = u - v;
   for i =1 :m
     if(abs(a(i)) < epsilon)
        a(i) = 0;
     end
   end

fprintf('T= %g\n',x(2*m+n+3));
   sumxi = sum(abs(xi));
   fprintf('Sum xi : %g\n',sumxi);
%fprintf('%d\n',size(xi));
   if sumxi > 0
    if n < 10
     for ijk = 1 :n
       if (abs(xi(ijk)) > 0)
         fprintf(' %d %g\n',ijk,xi(ijk));
       end;
     end;
    end;
   end;

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif round(RLPS.method) == 5
%input('before construct 2 BALL SPLIT wait');
    [n, m]= size(X);
fprintf('2BALL, SPLIT CONSTRAINTS\n');
fprintf('input size %d %d\n',n,m);
fprintf('RLPC sigMethod = %d %g\n',RLPS.sigMethod, RLPS.delta);
    epsilon = 1e-20;
    I = speye(n); 
    % Construct the  matrix
    fprintf('Constructing ...\n');
    H = zeros(n, 2*m +n+4); 
    for i=1:n
     if (RLPS.sigMethod < 0.5)
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, 0,0];%-Y(i)*RLPS.sigma(i)];
     else
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1, 0,0];%-Y(i)];  
     end
%      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), 1 -1];  
    end
%input('after fill H wait');
    b = sparse(ones(2*m,1));
    b = [b;C*ones(n,1)];  
    b = [b;0;0;0;0]; 
%    f(2*m+1) = 0;

% The inequality constraints      
    A = -sparse(H);
clear H;
clear I;
%input('after clear H wait');
    c = -sparse(1 * ones(n,1)); % THE MAGIC 1 constant
    K.l = n;
% WE SHOULDN"T need any more of these "non-negative"
%constraints, as sedumi will auto-do that.  HOWEVER
%sedumi does matrix size checking and it has to be
% length(b) = min(size(A)).
% the non-negativity constraints of u,v
   A = [A; -speye(2*m), zeros(2*m,n+4)];
   c=  [c; zeros(2*m,1)];
   K.l = K.l + 2*m; 
%input('after uv wait');
% the non-negativity constraints of slack
   A = [A; zeros(n,2*m), -speye(n), zeros(n,4)];
   c = [c;zeros(n,1)];
   K.l = n + K.l;

% the free variable b
  A = [A; zeros(1,2*m + n), -1, 0, 0,0];
  c = [c;0];
  K.l = K.l + 1;
  A = [A; zeros(1,2*m + n), 0, -1, 0,0];
  c = [c;0];
  K.l = K.l +1;
%%%%%
% now have complete new set of constraints for cone.
    H = zeros(n, 2*m +n+4); 
    for i=1:n
     if (RLPS.sigMethod < 0.5)
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), zeros(1,n), 0, 0, -Y(i)*RLPS.sigma(i),0];
     else
      Tn = 0;
      Tp = -Y(i);
      if (Y(i) < 0)
        Tn = -Y(i);
        Tp = 0;
      end;
      H(i,:)  = Y(i)*[ X(i,:), -X(i,:), zeros(1,n), 0, 0, Tp, Tn];%-Y(i)];
     end
    end
  A = [A; -sparse(H)];
  c = [c;zeros(n,1)];
  K.l = K.l + n;
clear H;
% the quadratic cone FOR sigmaPOS
  A = [A;zeros(1,2*m + n + 2), -1,0]; 
  if (RLPS.sigMethod < 0.5)
    A = [A; -speye(m), speye(m), zeros(m,n+4)];
  else
    zap = RLPS.delta * spdiags(RLPS.sigma ,zeros(1,1),m,m);
%[za,zb] = size(zap);
%fprintf('diagsigma %d %d %d\n',za,zb,m);

    A = [A; -zap, zap, zeros(m,n+4)];
    clear zap;
  end
  c = [c; 0;zeros(m,1)];
%  K.q = m +1;

% quadradic cone for sigmaNEG
  A = [A;zeros(1,2*m + n + 2), 0,-1]; 
  if (RLPS.sigMethod < 0.5)
    A = [A; -speye(m), speye(m), zeros(m,n+4)];
  else
    zap = RLPS.delta * spdiags(RLPS.sigmaNeg ,zeros(1,1),m,m);
%[za,zb] = size(zap);
%fprintf('diagsigma %d %d %d\n',za,zb,m);

    A = [A; -zap, zap, zeros(m,n+4)];
    clear zap;
  end
  c = [c; 0;zeros(m,1)];
  K.q = [m+1, m+1];
fprintf('K.q %d ', K.q);
    % Solve the Optimisation Problem
    
%fprintf('final sizes\n');
%[rr,cc] = size(A);
%fprintf('A %d %d\n',rr,cc);
%[rr,cc] = size(b);
%fprintf('b %d %d\n',rr,cc);
%[rr,cc] = size(c);
%fprintf('c %d %d\n',rr,cc);
%fprintf('%d %d\n', min(size(A)), length(b));

   pars.fid = 0; % quiet sedumi.
   [y,x,info]= sedumi(A,-b,c,K, pars);
fprintf('sedumi done pinf=%d dinf=%d numerr=%d feas=%g\n',info.pinf, info.dinf, info.numerr, info.feasratio);
   x = real(x);
   u = x(1:m);
   v = x(m+1:2*m);
   xi = x(2*m+1:2*m + n);
   bias = x(2*m + n + 1)-x(2*m+n+2);

   a = u - v;
   for i =1 :m
     if(abs(a(i)) < epsilon)
        a(i) = 0;
     end
   end

fprintf('T= %g\n',x(2*m+n+3));
   sumxi = sum(abs(xi));
   fprintf('Sum xi : %g\n',sumxi);
%fprintf('%d\n',size(xi));
   if sumxi > 0
    if n < 10
     for ijk = 1 :n
       if (abs(xi(ijk)) > 0)
         fprintf(' %d %g\n',ijk,xi(ijk));
       end;
     end;
    end;
   end;


%%%%%%%%% 
else 
  fprintf('Unsupported solve method in rlpc.m\n');
end; 

end    



