% ROBUST LIKNON, a Matlab program.  Leslie Grate 2003-2004

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (c) 1997-2004 The Regents of the University of California.
% All rights reserved.
% Permission is hereby granted, without written agreement and without
% license or royalty fees, to use, copy, modify, and distribute this
% software and its documentation for any purpose, provided that the above
% copyright notice and the following two paragraphs appear in all copies
% of this software.
%
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
% FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
% THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
% SUCH DAMAGE.
%
% THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
% PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% THIS MATLAB PROGRAM MUST HAVE SOME
% variables set before invoking.

%%%  MANDATORY VARS
% C = 0.01
% databasename = 'b1vb2'
% datadir = 'input'
% doLP = 1
% doSOCP = 1

%%%  HIGHLY SUGGESTED, overide internal defaults.
% doLOO = 1
% rhoarray = [0.1 0.5 1 2]
% NAMESFILE = 'input/genenames.txt'
% RLPS.method = 2;
% RLPS.sigMethod = 1;

%%%  EXTRA FUNCTIONALITY
% doPREDICT = 1
% predictfile = 'input/predictdata.dat'
% predictExpNames = ''
% doTEST = 1
% testfile = 'input/testdata.dat';
% ROTATEDATA = 1
% FACTOR = 14.7
% SMALLVALUE = 1e-10
% databasesignfilename = 'input/experimentLabels.txt'
% INFO = 1

if exist('SMALLVALUE','var') < 1
  SMALLVALUE = 1e-8; % threshold used to zero out small values.
  % this thresholding should be done better, with auto-scaling
  % of the data.  thresholding should be considered a hack
  % because the solution vector is not re-solved after being zeroed.
end;
if exist('cost','var') > 0
  if exist('C','var') < 1
    C = cost;
  end;
end;
if (exist('C','var') < 1) || (C <= 0)
  error('variable C must be set with a positive value\n');
end;
if (exist('datadir','var') < 1) || (strcmp(datadir, '') == 1)
  error('variable datadir must be set to a string where the data files reside\n');
end;
if (exist('databasename','var') < 1) || (strcmp(databasename, '') == 1)
  error('variable databasename must be set to the string name of the data file, no suffix or directory\n');
end;
if (exist('doLP','var') < 1) && (exist('doSOCP','var') < 1)
  error('one of doLP or doSOCP must be set greater than zero.\n');
end;

if exist('rhoarray','var') < 1
  rhoarray = [1 0.1 0.01];
  fprintf('Using default rhoarray (you should consider setting it) : %f.\n',rhoarray);
end;

if exist('doTEST','var') < 1
  doTEST = 0;
else
  if exist('testfile','var') < 1
  error('For doTEST, must have a testfile defined\n');
  end;
end;

if exist('doPREDICT','var') < 1
  doPREDICT = 0;
else
  if exist('predictfile','var') < 1
  error('For doPREDICT, must have a predictfile defined\n');
  end;
end;

if exist('ROTATEDATA','var') < 1
  ROTATEDATA = 0;
end;

dictthresh = 0;

% the "robust liknon problem structure" contains the noise
% data (sigma), solve method and parameters.

% the switch between pergene-noise (1+), or single radius (0)
if exist('sigMethod','var') > 0
  % already set.
  RLPS.sigMethod = sigMethod;
else
  if exist('RLPS','var') & isfield(RLPS, 'sigMethod') > 0
% already set.
    sigMethod = RLPS.sigMethod;
  else
    sigMethod = 1;
    RLPS.sigMethod = sigMethod;
  end;
end;
if exist('RLPS','var') & isfield(RLPS, 'sigMethod') > 0
% already set.
  sigMethod = RLPS.sigMethod;
else
  RLPS.sigMethod = 1;
  sigMethod = RLPS.sigMethod;
end;
if isfield(RLPS, 'method') > 0
% already set.
else
  RLPS.method = 2;   %default to splitt set to 1
end;
   RLPS.method    |         RLPS.sigMethod  
%%   solve method | spherical | per-gene noise | 2 Ball noise
% spherical   1   |   0       |    x           |     x     
% split T     2   |   x       |    1,2,3       |     x     
% split       3   |   x       |    1,2,3       |     x     
% 2 Ball      5   |   x       |    x           |     4     

%%% Now check for consistency.  we only support some
% solving methods with certain noise models.
if RLPS.method == 1 %spherical
  if RLPS.sigMethod > 0
    fprintf('Spherical method must have spherical noise,\n');
  end;
  RLPS.sigMethod = 0;
end;
if RLPS.method == 2 %split t set to one.
  if RLPS.sigMethod < 1
    fprintf('Warning: Split T method using spherical noise??\n');
  end;
  if (RLPS.sigMethod > 3.5) % & (RLPS.sigMethod < 2.5)
    fprintf('Split T method can not use 2 Ball noise.\n');
    RLPS.sigMethod = 1;
  end;
end;
if RLPS.method == 3 % split , same situation as split T
  if RLPS.sigMethod < 1
    fprintf('Warning: Split method using spherical noise??\n');
  end;
  if (RLPS.sigMethod > 3.5) % & (RLPS.sigMethod < 2.5)
    fprintf('Split method can not use 2 Ball noise.\n');
    RLPS.sigMethod = 1;
  end;
end;
if RLPS.method == 5 % 2 ball split
  if (RLPS.sigMethod > 3.5) % & (RLPS.sigMethod < 2.5)
  else
    fprintf('2 Ball Split method must use 2 Ball noise.\n');
    RLPS.sigMethod = 4;
  end;
end;
sigMethod = RLPS.sigMethod;
 
if exist('NAMESFILE','var') > 0
else
  NAMESFILE = ''; %gene names file.
end;
if exist('FACTOR','var') > 0
  fprintf('User set data scaling FACTOR in use: %g\n', FACTOR);
else
  FACTOR = 1;  %data values are multiplied by this.
end;
if exist('INFO','var') > 0
else
  INFO = 0;
end;
if exist('databasesignfilename','var') > 0
else
  databasesignfilename='';
end;

fprintf('sigMethod = %d\n',sigMethod);
fprintf('SMALLVALUE= %g\n',SMALLVALUE);
%fprintf('ndims %d\n',ndims(sigMethod));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Loading data file...\n');
% The data file is assumed to be very wide, genes in columns,
% experiments in rows.  The last column is the label of the
% experiment, +1 or -1.
tr = strcat(datadir,'/', databasename,'.dat');
trdata = load(tr);
if ROTATEDATA > 0
  trdata = trdata';
  fprintf('ROTATEDATA is set, data rotated.\n');
end;

[m,n] = size(trdata);  %m is number of samples, n-1 is number of genes.
numgenes = n-1;
if strcmp(databasesignfilename,'') < 1
  numgenes = n;
  signdata = load(databasesignfilename);
  [sm,sn] = size(signdata);
  if (sm == 1) 
    %has 1 row.
    signdata = signdata';
    sngenes = sn;
  elseif (sn == 1)
    %has 1 col
    sngenes = sm;
  end;
  if (sngenes ~= numgenes) 
    error('File of class labels does not have same number of items as number of genes. %d %d\n',sngenes,numgenes);
  end;
  y = signdata; %is a 1 column matrix.
else
  y = trdata(:,n);        %only the +1/-1 class labels.
end;

dumb = ones(numgenes,1);
xin = trdata(:,1:numgenes);  %throw away the labels if necessary

clear trdata;

xin = FACTOR * xin;

fprintf('Number of genes (columns) %d, number of samples (rows) %d\n',numgenes, m);
numposexamples = size(find(y == +1),1);
numnegexamples = size(find(y == -1),1);
fprintf('Number of Positive samples: %d, number of Negative samples: %d\n',numposexamples, numnegexamples);
if (numposexamples + numnegexamples ~= m)
  fprintf('ERROR, all class labels MUST be either +1 or -1\n');
  exit;
end;
if INFO > 0
  exit;
end;

%Make up the output directory name, create it and
%set some other file names.
str = strcat(databasename,'C',num2str(C));
stat = mkdir(str);
fstr = strcat(str,'/lpstat.dat') 
fout =fopen(fstr,'w');


if doTEST > 0
  testdata=load(testfile);
  [testm,testn] = size(testdata);
  testx = FACTOR * testdata(:,1:testn-1);
  testy = testdata(:,testn);
  fprintf('TEST data, number of genes %d, number of samples %d\n', testn-1, testm);
end;

if doPREDICT > 0
  predictdata=load(predictfile);
  [predictm,predictn]=size(predictdata);
  predictx = predictdata(:,1:predictn-1);
  predicty = predictdata(:,predictn);
  clear predictdata;
  predictx = FACTOR * predictx;
  fprintf('PREDICT data, number of genes %d, number of samples %d\n', predictn-1, predictm);
end;

fprintf('doing test: %d, doing predict: %d\n',doTEST,doPREDICT);

errtest = -1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% START of real stuff, Nominal Liknon, LP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if doLP > 0.5
      fprintf('LP start at %s\n',datestr(now));
  [anom,bnom] = lpc(xin,y,C);
      fprintf('LP end   at %s\n',datestr(now));
  [anom, zeroedNum] = threshold(anom, SMALLVALUE);
  lperr = lpcerror(xin,y,anom,bnom);

  % open file for holding the non-zero coeffcients for
  % this LP solution and write them out.
  outdatafileA = strcat(str,'/lpres','0','.txt');
  fouta = fopen(outdatafileA,'w');
  fprintf(fouta,'Value\t0\n');
  fprintf(fouta,'b\t%.10e\n',bnom);
      fprintf('LP raw b\t%.10e\n',bnom);
  denom = sqrt(dot(anom,anom));
  if denom > 1e-10
    fprintf('B %g\n',bnom/denom);
  else
    fprintf('magnitude of A is zero %g\n',denom);
  end;
  numina = 0;
  for ii=1 : length(anom)
    %only note values that are not "zero"
    if (abs(anom(ii)) > 1e-18) %basically a zero check.
      fprintf(fouta,'%d\t%.10e\n',ii,anom(ii));
      numina = numina + 1;
    end
  end
  fclose(fouta);
  fprintf('LP number of genes in solution: %d\n', numina);

  % Compute normal errors.  the number of genes in the solution is
  % length(find(anom)), the number of non-zero coefficients.
  errtest = -1;
  if doTEST > 0
    errtest = lpcerror(testx, testy, anom,bnom);
  end;
  fprintf(fout, '%g\t %g\t%g\n', lperr,errtest,length(find(anom)));
  fprintf('LP solution made: %g errors, and %g errors on test set\n', lperr,errtest);

  if numina > 0 
    if doPREDICT > 0
      predictoutfn = strcat(str,'/lppredict','LP','.txt');
      rlppredict(predictx, predicty, anom,bnom,0, predictoutfn,predictExpNames);
    end;

  %Now, create a reduced x matrix of ONLY those non-zero coeffcients,
  %and if desired perform LOO testing on it.
    dict = find(dumb); 
    dict = fs(anom, dictthresh,dict);
    x = fex(xin,dict);
    if doLOO > 0.5
      RLPSforLP.sigma = zeros(m,1);
      RLPSforLP.sigmaNeg = zeros(m,1);
      RLPSforLP.sigmaMethod = 0;
      RLPSforLP.method = 0;
      RLPSforLP.delta = 1;
      loo(x,y,C,str, RLPSforLP, SMALLVALUE);
    end
    genelist(dict,NAMESFILE,0,str);
    clear x;
  end;
end  % of doLP
%%%%%%%%%%%%%%
fprintf('DONE LP\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now the SOCP part.  Robust Liknon
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if doSOCP > 0.5

  RLPS.sigmaNeg = zeros(numgenes,1);  %most methods don't use this.
  for k = 1 : length(rhoarray)
    rho = rhoarray(k);
    fprintf('SOCP C=%g rho=%g\n',C,rho);

    %FIRST compute Noise matrix (sigma)
    %compute data range for each measured gene.
    minx = min(xin);  %produces vector of minimums
    maxx = max(xin);  %produces vector of maximums
    diff = maxx - minx;
    minrange = min(diff); % finds smallest component of diff
    if abs(minrange) > 1e-10
      noisesig = rho * (minrange * sqrt(numgenes));
    else
      noisesig = 0;
    end;
    RLPS.delta = 1;  %an extra factor, should be set to 1. not supported anymore.
    if (sigMethod < 0.5)
      % Spherical Noise.  sigma matrix filled with all same values.
      fprintf('NoiseMethod: Spherical\n');
      RLPS.sigma = noisesig * ones(m,1);
      RLPS.sigmaNeg = zeros(size(RLPS.sigma,1),1);
    elseif (sigMethod < 1.5)
     % Noise per gene, each gene is set to rho * (its range)
      fprintf('NoiseMethod: PerGene data range\n');
      RLPS.sigma = rho * diff';
      RLPS.sigmaNeg = zeros(size(RLPS.sigma,1),1);
      RLPS.delta = 1;
    elseif (sigMethod < 2.5) 
      % Noise per gene, std deviation range way 
      fprintf('NoiseMethod: PerGene range standard deviation\n');
      RLPS.delta = 1;
      RLPS.sigma = rho * std(xin)'; % (makes sigma a row vector) then transpose to column.
    elseif (sigMethod < 3.5)
      % Noise per gene, std deviation global range way 
      fprintf('NoiseMethod: PerGene global range standard deviation\n');
      RLPS.delta = 1;
      RLPS.sigma = std(xin)'; % (makes sigma a row vector) then transpose to column.
      stdfac = sum(abs(diff)) / sqrt(dot(RLPS.sigma,RLPS.sigma));
      RLPS.sigma = rho * stdfac * RLPS.sigma;
    elseif (sigMethod < 4.5)
      % Noise per gene, positive and negative example balls (2 Ball),
      % so world is split into positive and negative examples, and
      % each gets their own min/max.
      fprintf('NoiseMethod: 2 Ball PerGene data range\n');
      RLPS.delta = 1;
      pi = 0;
      ni = 0;  %first find indexes of one positive, and negative example.
      for ii=1:m
        if y(ii) > 0 
          pi = ii;
        else
          ni = ii;
        end;
      end;   
      for ii=1:m  %now create min and max vectors for the positive and negative worlds.
        if y(ii) > 0 
          minpx = min(xin(ii,pi),xin(ii,:));
          maxpx = max(xin(ii,pi),xin(ii,:));
        else
          minnx = min(xin(ii,ni),xin(ii,:));
          maxnx = max(xin(ii,ni),xin(ii,:));
        end;
      end;   
      diffp = maxpx - minpx;
      diffn = maxnx - minnx;
      RLPS.sigma = rho * diffp';    %the positive example sigma vector
      RLPS.sigmaNeg = rho * diffn'; %the negative example sigma vector
    end
    % END OF compute noise matrix

    %% NOW, SIGMA MATRIX COMPLETE,
    %% GO CREATE AND SOLVE ROBUST LIKNON PROBLEM.
    fprintf('SOCP start %s\n',datestr(now));
    [arob, brob]= rlpc(xin,y,C, RLPS);
    fprintf('SOCP end   %s\n',datestr(now));

%    fprintf('sqrt(wtsw) %g, sqrt(wts2w) %g\n',sqrt(arob' * diag(RLPS.sigma) * arob), sqrt(arob' * diag(RLPS.sigma .* RLPS.sigma) * arob));
    fprintf('b\t%.10e\n',brob);
    denom = sqrt(dot(arob,arob));
    fprintf('magnitude of A %g\n',denom);
    if denom > 1e-10
      fprintf('B %g\n',brob/denom);
    else
      fprintf('magnitude of A is zero %g\n',denom);
    end;
    % This is where the "really small values" are forced to zero.
    [arob, zeroedNum] = threshold(arob, SMALLVALUE);
    
    % Now write data file containing coefficients for the
    % non-zero genes of this robust solution.
    numina = 0;
    outdatafileA = strcat(str,'/lpres',num2str(rho),'.txt');
    fouta = fopen(outdatafileA,'w');
    fprintf(fouta,'Value\t0\n');
    fprintf(fouta,'b\t%.10e\n',brob);
    for ii=1 : length(arob)
      if (abs(arob(ii)) > 1e-18) %basically a zero check
        fprintf(fouta,'%d\t%.10e\n',ii,arob(ii));
        numina = numina + 1;
      end
    end
    fclose(fouta);
    fprintf('SOCP number of genes in solution: %d (number set to zero %d)\n', numina, zeroedNum);

    % make a reduced data matrix containing only data for non-zero
    % genes.  used later in LOO and making gene list, 
    % but you might also use it for computing errors
    if numina > 0 
      dict = find(dumb); 
      dict = fs(arob, dictthresh,dict);
      x = fex(xin,dict);  %the reduced data values matrix
    end;
    % NOW COMPUTE SOME ERROR STATS
    errtr = lpcerror(xin,y,arob,brob); %NORMAL ERROR OF socp solution
    nomerr = [-1;-1];
    if exist('anom','var') > 0
      nomerr = wlpcerror(xin,y,anom,bnom, RLPS); % worst case error of LP solution
    end
    wcerr = wlpcerror(xin,y,arob,brob, RLPS); % worst case error on this SOCP solution
    if doTEST > 0
      errtest = lpcerror(testx, testy, arob,brob);
      wcerrtest = wlpcerror(testx, testy, arob, brob, RLPS); 
    else
      errtest = -1;
      wcerrtest = [-1;-1];
    end;
    fprintf('SOCP solution made: %g(intersects %g) errors, and %g(intersects %g) errors on test set\n', wcerr(1),wcerr(2), wcerrtest(1), wcerrtest(2));
    fprintf(fout, '%g(%g)\t%g\t%g(%g)\t%g(%g)\t%g\t%g(%g)\t%d\t%g\t%d\t%d\n', noisesig,rho,errtr,nomerr(1),nomerr(2),wcerr(1),wcerr(2),errtest,wcerrtest(1),wcerrtest(2), length(find(arob)), minrange, numgenes,RLPS.sigMethod);
    fprintf( '%g(%g)\t%g\t%g(%g)\t%g(%g)\t%g\t%g(%g)\t%d\t%g\t%d\t%d\n', noisesig,rho,errtr,nomerr(1),nomerr(2),wcerr(1),wcerr(2),errtest,wcerrtest(1),wcerrtest(2), length(find(arob)), minrange, numgenes,RLPS.sigMethod);

    if numina > 0
      genelist(dict,NAMESFILE,rho,str);

      if doLOO > 0.5
        loo(x,y,C,str, RLPS, SMALLVALUE);
      end

      clear x;
    end;
  end

end %doSOCP
